#!/usr/bin/perl -w

package RPM::Source::Macros;
use warnings;
use Carp;

sub new {
    my $class = shift;
    my $arg={
	SPECFILE=> undef,
	VERBOSE=>0,
	MACRO=>{},
	@_
	};
    print STDERR "entering RPM::Source::Macro::new\n" if $arg->{VERBOSE}>2;
    my $macro=$arg->{MACRO};
    $macro->{'nil'}='';
    $macro->{'homedir'}=$ENV{'HOME'};
    $macro->{'_topsrcdir'}='%_topdir';
    $macro->{'_sourcedir'}="%_topsrcdir/SOURCES";
    $macro->{'_specdir'}="%_topsrcdir/SPECS";
    $macro->{'_srcrpmdir'}="%_topdir/SRPMS";
    $macro->{'_topdir'}="$ENV{'HOME'}/RPM";
    if (-e $ENV{'HOME'}.'/.rpmmacros') {
	open (RPMMACROS, $ENV{'HOME'}.'/.rpmmacros') || die "can't open .rpmmacros:$!";
	while (<RPMMACROS>) {
	    if (/^[%]([\w_\/.\d]+)\s+(.*\S)\s*$/) {
		$macro->{$1}=$2;
	    }
	}
	close (RPMMACROS);
    }
    if (defined $arg->{SPECFILE}) {
	open (SPECFILE, $arg->{SPECFILE}) || die "can't open spec $arg->{SPECFILE}: $!";
	my @spec_lines=<SPECFILE>;
	&__raw_process_spec($macro, \@spec_lines);
	close (SPECFILE);
    } elsif (defined $arg->{SPECLIST} and ref $arg->{SPECLIST}) {
	&__raw_process_spec($macro, $arg->{SPECLIST});
    }
    bless $macro, $class;
    return $macro;
}

sub __raw_process_spec {
    my $macro=shift;
    my $specarray=shift;
    my $multilinedefine;
    my $multilinebody;
    foreach (@$specarray) {
	chomp;
	# to avoid recursion Name: %name; old hack is && not $macro->{'version'};
	$macro->{'name'}=&macro_subst($macro, $1) if /^name\s*:\s*(.*\S)\s*$/i;
	$macro->{'version'}=&macro_subst($macro, $1) if /^version\s*:\s*(.*\S)\s*$/i;
	$macro->{'release'}=&macro_subst($macro, $1) if /^release\s*:\s*(.*\S)\s*$/i;
	$macro->{'epoch'}=&macro_subst($macro, $1) if /^epoch\s*:\s*(.*\S)\s*$/i;
	$macro->{'serial'}=&macro_subst($macro, $1) if /^serial\s*:\s*(.*\S)\s*$/i;
	
	if ($multilinedefine) {
	    if (s/\\$//) {
		$multilinebody.=$_."\n";
	    } else {
		$multilinebody.=$_."\n";
		$macro->{$multilinedefine}=$multilinebody;
		undef $multilinedefine;
	    }
	}
	if (/^\s*[%](?:define|global)\s+([\w_\d]+)(?:\(\))?\s+(.*)\\$/) {
	    $multilinedefine=$1;
	    $multilinebody=$2;
	} elsif (/^\s*[%](?:define|global)\s+([\w_\d]+)(?:\(\))?\s+(.+?)\s*$/) {
	    $macro->{$1}=$2;
	}
    }

    foreach my $key  
	(qw/
     _topdir
     _topsrcdir
     _sourcedir
     _specdir
     _srcrpmdir
     name
     version
     release
     /) {
	    if (! defined $macro->{$key}) {
		print STDERR "WARNING: undefined $key!\n";
	    } else {
		&macro_subst_inplace($macro,\$macro->{$key});
	    }
    }
}

sub macro_subst_defined {
    my $macroptr=shift;
    my $arg=shift;
    &macro_subst_inplace($macroptr, \$arg, 1);
    #print STDERR "macro subst: $arg\n";
    return $arg;
}

sub macro_subst {
    my $macroptr=shift;
    my $arg=shift;
    &macro_subst_inplace($macroptr, \$arg, 0);
    #print STDERR "macro subst: $arg\n";
    return $arg;
}


sub macro_subst_inplace {
    my $macroptr=shift;
    my $argptr=shift;
    my $subst_only_defined=shift;
    #print STDERR "subst: $$argptr\n";
    Carp::cluck "undef" unless defined $$argptr;

    my $oldval;
    do {
	$oldval=$$argptr;
	#print STDERR ":: $$argptr\n";
	$$argptr=~s/\%\{(\!)?\?([\w_\d]+)(?::([\w_\.\d]+))?\}?/&__raw_subst_if($macroptr,$1,$2,$3)/eg;
	if ($subst_only_defined) {
	    $$argptr=~s/(\%\{?([\w_\d]+)\}?)/&__raw_subst($macroptr,$2,$1)/eg;
	} else {
	    $$argptr=~s/(\%\{?([\w_\d]+)\}?)/&__raw_subst($macroptr,$2,'')/eg;
	}
    } while ($oldval ne $$argptr);
}

sub __raw_subst {
    my $macroptr=shift;
    my $name=shift;
    my $orig=shift;
    my $val = $macroptr->{$name};
    #print STDERR "__raw_subst ($orig) $name=[$val]\n";
    return defined $val ? $val : $orig;
}

sub __raw_subst_if {
    my $macroptr=shift;
    my ($not,$cond,$arg)=@_;
    #print STDERR "__raw_if $not $cond =[$arg]\n";
    return $arg if ($not xor $macroptr->{$cond});
    return '';
}

1;

=head1	NAME

Test::Repocop::SQL -   for repocop

=head1	DESCRIPTION

TODO

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2008 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

