#!/usr/bin/perl -w

package RPM::Source::Transform::ALTLinuxBackport;
use strict;
use warnings;
use Carp;
use Getopt::Long;
use RPM::Source::Transform;
our @ISA=q/RPM::Source::Transform/;

$RPM::Source::Transform::nextrel='altbackport';
$RPM::Source::Transform::NEXTREL_METHOD{'altbackport'}=\&nextrel_altbackport;
#$RPM::Source::Transform::INITREL_METHOD{'altbackport'}=\&nextrel_altbackport_init;
$RPM::Source::Transform::defaultchangelog='- backport';
$RPM::Source::Transform::force_transform_mode=1;


our $backport_class='M';
our $backport_branch='51';

sub process_options {
    my ($self)=@_;
    $self->SUPER::process_options();
    $RPM::Source::Transform::defaultchangelog='- '.$self->get_backport_suffix().' backport';
}

sub get_backport_class() {
    my ($self)=@_;
    return $backport_class;
}

sub set_backport_class() {
    my ($self,$class)=@_;
    $backport_class=$class;
}

sub get_backport_branch() {
    my ($self)=@_;
    return $backport_branch;

}

sub set_backport_branch() {
    my ($self,$branch)=@_;
    $backport_branch=$branch;
}

sub get_backport_suffix() {
    my ($spec)=@_;
    return $spec->get_backport_class().$spec->get_backport_branch();
}

sub nextrel_altbackport {
    my ($rel,$spec,$parent)=@_;
    #my $backportsuffix=$spec->get_backport_suffix();
    # $rel!~/\%{?release}?/; otherwise it is recurrent release invocation
    $spec->macros()->macro_subst_inplace(\$rel) if $rel=~/\%/ and $rel!~/\%{?release}?/;
    return &_nextrel_altbackport_raw($rel, $backport_class.$backport_branch);
}

# sub nextrel_altbackport_init {
#     #my ($rel,$spec,$parent)=@_;
#     my ($spec,$parent)=@_;
#     my $rel=$spec->get_section('package','')->get_tag('Release');
#     #my $backportsuffix=$spec->get_backport_suffix();
#     return &_nextrel_altbackport_raw($rel, $backport_class.$backport_branch);
# }

sub _nextrel_altbackport_raw {
    my ($rel, $backportsuffix)=@_;
    die "bad Release: $rel (does not start with alt - not an ALT Linux release !)" unless $rel=~/^(?:alt|\%)/;
    die "bad Release: $rel (does not have a digit - not an ALT Linux release !)" unless $rel=~/^(?:alt.*\d|\%).*$/;
    my $newrel = 'alt0.'.$backportsuffix.'.1';
    my ($relmajor,$relminor);
    if ($rel=~/^alt(.+\.|)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
    } elsif($rel=~/^alt(.+)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
    }
    if ($relminor) {
	$newrel = 'alt'.$relmajor.($relminor-1).'.'.$backportsuffix.'.1';
    }
    return $newrel;
}


1;
